from typing import Literal


class Proxy:
    socks5 = "SOCKS5"
    socks4 = "SOCKS4"
    http = "HTTP"
    https = "HTTPS"


class Market:
    class Category:
        _Literal = Literal["steam", "fortnite", "mihoyo", "riot", "telegram", "supercell", "origin", "world-of-tanks", "wot-blitz", "epicgames", "gifts", "escape-from-tarkov", "socialclub", "uplay", "war-thunder", "discord", "tiktok", "instagram", "battlenet", "vpn", "cinema", "roblox", "spotify", "warface", "minecraft"]
        steam = "steam"
        fortnite = "fortnite"
        mihoyo = "mihoyo"
        riot = "riot"
        telegram = "telegram"
        supercell = "supercell"
        origin = "origin"
        wot = "world-of-tanks"
        wot_blitz = "wot-blitz"
        epicgames = "epicgames"
        gifts = "gifts"
        eft = "escape-from-tarkov"
        socialclub = "socialclub"
        uplay = "uplay"
        war_thunder = "war-thunder"
        discord = "discord"
        tiktok = "tiktok"
        instagram = "instagram"
        battlenet = "battlenet"
        vpn = "vpn"
        cinema = "cinema"
        roblox = "roblox"
        spotify = "spotify"
        warface = "warface"
        minecraft = "minecraft"

    class CategoryId:
        _Literal = Literal[1, 3, 4, 5, 7, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 22, 23, 24, 26, 27, 28, 30, 31]
        steam = 1
        fortnite = 9
        mihoyo = 17
        riot = 13
        telegram = 24
        supercell = 15
        origin = 3
        wot = 14
        wot_blitz = 16
        epicgames = 12
        gifts = 30
        eft = 18
        socialclub = 7
        uplay = 5
        war_thunder = 27
        discord = 22
        tiktok = 20
        instagram = 10
        battlenet = 11
        vpn = 19
        cinema = 23
        roblox = 31
        spotify = 26
        warface = 4
        minecraft = 28

    class OperationTypes:
        _Literal = Literal["income", "cost", "refilled_balance", "withdrawal_balance", "paid_item", "sold_item", "money_transfer", "receiving_money", "internal_purchase", "claim_hold", "bid"]
        income = "income"
        cost = "cost"
        refilled_balance = "refilled_balance"
        withdrawal_balance = "withdrawal_balance"
        paid_item = "paid_item"
        sold_item = "sold_item"
        money_transfer = "money_transfer"
        receiving_money = "receiving_money"
        internal_purchase = "internal_purchase"
        claim_hold = "claim_hold"
        bid = "bid"

    class HoldPeriod:
        _Literal = Literal["hour", "day", "week", "month", "year"]
        hour = "hour"
        day = "day"
        week = "week"
        month = "month"
        year = "year"

    class Currency:
        _Literal = Literal["cny", "usd", "rub", "eur", "uah", "kzt", "byn", "gbp"]
        cny = "cny"
        usd = "usd"
        rub = "rub"
        eur = "eur"
        uah = "uah"
        kzt = "kzt"
        byn = "byn"
        gbp = "gbp"

    class ItemOrigin:
        _Literal = Literal["brute", "stealer", "fishing", "autoreg", "personal", "resale", "dummy", "self_registration"]
        brute = "brute"
        stealer = "stealer"
        fishing = "fishing"
        autoreg = "autoreg"
        personal = "personal"
        resale = "resale"
        dummy = "dummy"      # Only for steam (Пустышки)
        self_reg = "self_registration"

    class Guarantee:
        _Literal = Literal[-1, 0, 1]
        half_day = -1
        day = 0
        three_days = 1

    class ItemStatus:
        _Literal = Literal["active", "paid", "deleted", "awaiting", "closed", "discount_request", "stickied", "pre_active"]
        active = "active"
        paid = "paid"
        deleted = "deleted"
        awaiting = "awaiting"
        closed = "closed"
        discount_request = "discount_request"
        stickied = "stickied"
        pre_active = "pre_active"

    class ItemOrder:
        _Literal = Literal["price_to_up", "price_to_down", "pdate_to_down", "pdate_to_up", "pdate_to_down_upload", "pdate_to_up_upload", "ddate_to_down", "ddate_to_up", "edate_to_down", "edate_to_up"]
        cheap = "price_to_up"
        expensive = "price_to_down"
        newest = "pdate_to_down"
        oldest = "pdate_to_up"
        newest_upload = "pdate_to_down_upload"
        oldest_upload = "pdate_to_up_upload"
        newest_deleted = "ddate_to_down",
        oldest_deleted = "ddate_to_up"
        newest_edited = "edate_to_down",
        oldest_edited = "edate_to_up"

    class AppID:
        _Literal = Literal[1, 730, 578080, 753, 570, 440, 252490, 304930, 232090, 322330]
        ALL = 1
        CS2 = 730
        PUBG = 578080
        Steam = 753
        Dota = 570
        TF2 = 440
        Rust = 252490
        Unturned = 304930
        KF2 = 232090
        DST = 322330


class Forum:
    class Contests:
        class Length:
            _Literal = Literal["minutes", "hours", "days"]
            minutes = "minutes"
            hours = "hours"
            days = "days"

        class UpgradePrize:
            _Literal = Literal[1, 6, 12, 14, 17, 19]
            supreme = 1
            legend = 6
            antipublic = 12
            uniq = 14
            photo_leaks = 17
            auto_participation = 19

    class ThreadOrder:
        _Literal = Literal["natural", "natural_reverse", "thread_create_date", "thread_create_date_reverse", "thread_update_date", "thread_update_date_reverse", "thread_view_count", "thread_view_count_reverse", "thread_post_count", "thread_post_count_reverse", "first_post_likes", "first_post_likes_reverse"]
        default = "natural"
        oldest = "thread_create_date"
        newest = "thread_create_date_reverse"
        newest_bumped = "thread_update_date"
        oldest_bumped = "thread_update_date_reverse"
        min_views = "thread_view_count"
        max_views = "thread_view_count_reverse"
        min_posts = "thread_post_count"
        max_posts = "thread_post_count_reverse"
        min_likes = "first_post_likes"
        max_likes = "first_post_likes_reverse"

    class PostOrder:
        _Literal = Literal["natural", "natural_reverse", "post_likes", "post_likes_reverse"]
        default = "natural"
        default_reverse = "natural_reverse"
        max_likes = "post_likes"
        min_likes = "post_likes_reverse"

    class ReplyGroups:
        _Literal = Literal[0, 2, 21, 22, 23, 60, 351]
        staff = 0      # КФ + кураторы
        everyone = 2   # Все
        local = 21     # Местный и выше
        resident = 22  # Постоялец и выше
        expert = 23    # Эксперт и выше
        guru = 60      # Гуру и выше
        ai = 351       # ИИ и выше

    class Arbitrage:
        class TransferType:
            _Literal = Literal["safe", "notsafe"]
            safe = "safe"
            not_safe = "notsafe"

        class Currency:
            cny = "cny"
            usd = "usd"
            rub = "rub"
            eur = "eur"
            uah = "uah"
            kzt = "kzt"
            byn = "byn"
            gbp = "gbp"


class Antipublic:
    class SearchBy:
        _Literal = Literal["email", "password", "domain"]
        email = "email"
        password = "password"
        domain = "domain"

    class SearchDirection:
        _Literal = Literal["start", "strict", "end"]
        start = "start"
        strict = "strict"
        end = "end"



"""
LOLZTEAM it's library that contains all the methods of the Lolzteam API (Market/Forum/Antipublic)

You can find full documentation here -> https://github.com/AS7RIDENIED/LOLZTEAM
"""

from .API import Forum
from .API import Market
from .API import Antipublic

from . import Exceptions
from . import Tweaks
from . import Constants
from .BBCODE import BBCODE



from .Tweaks import SendAsAsync, _MainTweaks
from .Models import Payment


async def CheckPayment(
    self,
    amount,
    comment: str = _MainTweaks.NONE,
    user_id: int = _MainTweaks.NONE,
    allow_hold: bool = _MainTweaks.NONE,
    include_payment_data: bool = False,
) -> Payment:
    """
    A function to check payment status based on given parameters.

    ## Status codes:
    - 0 - Payment not found
    - 1 - Paid
    - 2 - Paid by incorrect user
    - 3 - Paid in hold

    :param amount: The amount to check for payment. Not recomended to use it without comment
    :param comment: Optional comment for the payment
    :param user_id: Optional forum user_id to check payment
    :param allow_hold: Allow payments with hold. True to allow, False to disallow. Allowed by default
    :param include_payment_data: Include detailed payment data
    :return: a dictionary containing the status, status code, message, and payment data
    """
    from .API import Market
    if type(self) is not Market:
        raise TypeError("You need to pass Market object")
    payments = await SendAsAsync(
        self.payments.history,
        pmin=amount,
        pmax=amount,
        comment=comment,
        operation_type="income",
        is_hold=allow_hold,
    )
    payments_json = payments.json()
    response = {}
    if payments_json["payments"]:
        for payment in reversed(payments_json["payments"].values()):
            response["status"] = "success"
            response["status_code"] = 1
            response["message"] = "Paid"
            if include_payment_data:
                response["payment_data"] = payment
            if user_id:
                if payment["data"]["user_id"] != user_id:
                    response["status"] = "error"
                    response["status_code"] = 2
                    response["message"] = "Paid by incorrect user"
            if not allow_hold:
                if payment["is_hold"]:
                    response["status"] = "error"
                    if response["status_code"] != 1:
                        response["status_code"] = int(
                            str(response["status_code"]) + "3"
                        )
                        response["message"] += " | Payment in hold"
                    else:
                        response["status_code"] = 3
                        response["message"] = "Payment in hold"
            break
    else:
        response["status"] = "error"
        response["status_code"] = 0
        response["message"] = "Not found"
    response["parameters"] = {
        "amount": amount,
        "comment": comment,
        "user_id": user_id,
        "allow_hold": allow_hold,
    }
    response = Payment(**response)
    return response



import functools
import logging
import inspect
import asyncio
import ctypes
import base64
import types
import httpx
import json
import time
import sys
import re
import os

from . import Exceptions
from .API import version
from multiprocessing import Value, Lock
from binascii import Error as binasciiError
from concurrent.futures import ThreadPoolExecutor

_WarningsLogger = logging.getLogger("LOLZTEAM.Warnings")
_DebugLogger = logging.getLogger("LOLZTEAM.Debug")


class _MainTweaks:
    class NonePlaceholder:
        pass
    NONE = NonePlaceholder()

    @staticmethod
    def market_variable_fix(variable):
        if variable is None:
            variable = "nomatter"
        elif variable is True:
            variable = "yes"
        elif variable is False:
            variable = "no"
        return variable

    @staticmethod
    async def _auto_delay_async(self, delay: float = 3.0):
        """
        Sleep for time difference between the last call and current call if it's less than 3 seconds
        """
        delay += self.additional_delay
        # Возможно когда-нибудь моя любимка полюбит меня вновь и сделает, чтобы лолз возвращал время с миллисекундами и тогда мы избавимся от костыля в задержке ❤️❤️❤️
        if self.bypass_429:
            if self._delay_synchronizer:
                time_diff = time.time() - self._auto_delay_time.value
                if (
                    time_diff < delay
                ):
                    _DebugLogger.debug(
                        f"Sleeping for {round(delay-time_diff, 3)} seconds")
                    await asyncio.sleep(delay - time_diff)
            else:
                time_diff = time.time() - self._auto_delay_time
                if (
                    time_diff < delay
                ):
                    _DebugLogger.debug(
                        f"Sleeping for {round(delay-time_diff, 3)} seconds")
                    await asyncio.sleep(delay - time_diff)

    @staticmethod
    def setup_jwt(self, token):
        try:
            if "." in self._token:
                decoded_payload = json.loads(base64.b64decode(token.split(".")[1] + "==" if "==" not in token.split(".")[1] else token.split(".")[1]).decode("utf-8"))
                self.user_id = decoded_payload.get("sub", "me")
                scopes = decoded_payload.get("scope", "basic read post conversate market").split(" ")  # Tweak На самых первых jwt токенах не указывались скопы
                self._scopes = scopes
            else:
                raise Exceptions.BAD_TOKEN("Your token is invalid. You must check if you have pasted your token fully or create new token and use it instead")
        except (binasciiError, json.JSONDecodeError):
            raise Exceptions.BAD_TOKEN("Your token is invalid. You must check if you have pasted your token fully or create new token and use it instead")
        _DebugLogger.debug(f"Setuped jwt token | User ID: {self.user_id} | Scopes: {self._scopes}")

    @staticmethod
    def _CheckScopes(scopes: list = None):
        def _wrapper(func):
            @functools.wraps(func)
            def _wrapper_costyl(func_self, *args, **kwargs):
                main_self = func_self
                if hasattr(main_self, "__self__"):
                    main_self = main_self.__self__
                if hasattr(main_self, "_api"):
                    main_self = main_self._api
                path = str(func.__qualname__).replace(
                    "<class '", "").replace("'>", "")
                for bad_path, cute_path in {
                    "__Profile": "profile",
                    "__Payments": "payments",
                    "__Category": "category",
                    "__List": "list",
                    "__Publishing": "publishing",
                    "__Purchasing": "purchasing",
                    "__Managing": "managing",
                    "__Proxy": "proxy",
                    "__Steam": "steam",
                    "__Fortnite": "fortnite",
                    "__MiHoYo": "mihoyo",
                    "__Valorant": "valorant",
                    "__LeagueOfLegends": "lol",
                    "__Telegram": "telegram",
                    "__Supercell": "supercell",
                    "__Origin": "origin",
                    "__WorldOfTanks": "wot",
                    "__WorldOfTanksBlitz": "wot_blitz",
                    "__EpicGames": "epicgames",
                    "__EscapeFromTarkov": "eft",
                    "__SocialClub": "socialclub",
                    "__Uplay": "uplay",
                    "__WarThunder": "war_thunder",
                    "__Discord": "discord",
                    "__TikTok": "tiktok",
                    "__Instagram": "instagram",
                    "__BattleNet": "battlenet",
                    "__VPN": "vpn",
                    "__Cinema": "cinema",
                    "__Spotify": "spotify",
                    "__Warface": "warface",
                    "__Tag": "tag",
                    "__Categories": "categories",
                    "__Forums": "forums",
                    "__Pages": "pages",
                    "__Threads": "threads",
                    "__Posts": "posts",
                    "__Tags": "tags",
                    "__Users": "users",
                    "__Profile_posts": "profile_posts",
                    "__Conversations": "conversations",
                    "__Notifications": "notifications",
                    "__Search": "search",
                    "__Oauth": "oauth",
                    "__Posts_comments": "comments",
                    "__Contests": "contests",
                    "__Arbitrage": "arbitrage",
                    "__Money": "money",
                    "__Upgrade": "upgrade",
                    "__Avatar": "avatar",
                    "__Profile_posts_comments": "comments",
                    "__Conversations_messages": "messages",
                }.items():
                    if bad_path in path:
                        path = path.replace(bad_path, cute_path)
                for scope in scopes:
                    scope_parsed = scope.split("?")
                    joined = ", ".join(scope_parsed)
                    if len(scope_parsed) > 1:
                        if not any(
                            _scope in main_self._scopes for _scope in scope_parsed
                        ):
                            logging.warning(
                                msg=f'{Exceptions.MISSING_SCOPE.__name__}: One of [{joined}] scope is required to use "{path}" but not provided in your token.\nYou should recreate token with at least one of these scopes.',
                                stack_info=False,
                            )
                    elif scope_parsed[0] not in main_self._scopes:
                        logging.warning(
                            msg=f'{Exceptions.MISSING_SCOPE.__name__}: "{joined}" scope is required to use "{path}" but not provided in your token.\nYou should recreate token with "{joined}" scope.',
                            stack_info=False,
                        )
                return func(func_self, *args, **kwargs)

            return _wrapper_costyl

        return _wrapper

    def _RetryWrapper(func):
        async def _wrapper(*args, **kwargs):
            tries = 0
            self = kwargs["self"]
            if self._delay_synchronizer:
                self._lock.acquire()
            while tries < 30:
                tries += 1
                try:
                    response = await func(*args, **kwargs)
                    if self._delay_synchronizer:
                        self._lock.release()  # Unlocking to prevent softlock
                    return response
                except httpx.ConnectError as e:
                    if tries == 30:
                        if self._delay_synchronizer:
                            self._lock.release()  # Unlocking to prevent softlock
                        raise e
                    await asyncio.sleep(0.25)
                    continue
        return _wrapper

    def _TrimNONE(dictionary: dict) -> dict:
        for key, value in dictionary.copy().items():
            if isinstance(value, _MainTweaks.NonePlaceholder):
                dictionary.pop(key)
            if type(value) is dict:
                dictionary[key] = _MainTweaks._TrimNONE(value)
        return dictionary

    async def _ExecCode(func, loc, cur_kwargs):  # Смотрите на свой страх и риск. Никому не стоит сюда суваться
        SEND_AS_ASYNC = loc.get("SEND_AS_ASYNC", False)
        CREATE_JOB = loc.get("CREATE_JOB", False)
        loc["NONE"] = _MainTweaks.NonePlaceholder()
        self = functools.partial(func).func.__self__
        if "scope" in functools.partial(func).func.__code__.co_varnames:
            func = next((c for c in (c.cell_contents for c in func.__closure__) if isinstance(c, types.FunctionType)), None,)
        else:
            func = functools.partial(func).func
        if hasattr(self, "_api"):
            self = self._api
        loc["self"] = self  # Определяем селф в нашей локали, чтобы дальше не ебнуло

        arguments = func.__code__.co_varnames
        for arg in arguments:
            if arg != "self":
                exec(f"{arg} = NONE", loc)
            if arg == "kwargs":
                exec("kwargs = {}", loc)
        for arg, value in cur_kwargs.items():  # Чек на левые кварги
            if "kwargs" not in arguments and arg not in arguments:
                raise Exceptions.INVALID_ARG(
                    f'Function "{func.__name__}" don\'t have "{arg}" parameter'
                )
            if arg not in arguments:
                loc["kwargs"][arg] = value
            else:
                loc[arg] = value

        # Не придумал пока лучшего испольнения чека скопов
        if SEND_AS_ASYNC:
            pre_def = inspect.getsource(func).split("def")[0]
            pattern = r"\[.*?\]"
            result = re.findall(pattern, pre_def)
            scopes = None
            if result:
                scopes = eval(result[0])
            if scopes:
                path = str(func.__qualname__).replace(
                    "<class '", "").replace("'>", "")
                for bad_path, cute_path in {
                    "__Profile": "profile",
                    "__Payments": "payments",
                    "__Category": "category",
                    "__List": "list",
                    "__Publishing": "publishing",
                    "__Purchasing": "purchasing",
                    "__Managing": "managing",
                    "__Proxy": "proxy",
                    "__Steam": "steam",
                    "__Fortnite": "fortnite",
                    "__MiHoYo": "mihoyo",
                    "__Valorant": "valorant",
                    "__LeagueOfLegends": "lol",
                    "__Telegram": "telegram",
                    "__Supercell": "supercell",
                    "__Origin": "origin",
                    "__WorldOfTanks": "wot",
                    "__WorldOfTanksBlitz": "wot_blitz",
                    "__EpicGames": "epicgames",
                    "__EscapeFromTarkov": "eft",
                    "__SocialClub": "socialclub",
                    "__Uplay": "uplay",
                    "__WarThunder": "war_thunder",
                    "__Discord": "discord",
                    "__TikTok": "tiktok",
                    "__Instagram": "instagram",
                    "__BattleNet": "battlenet",
                    "__VPN": "vpn",
                    "__Cinema": "cinema",
                    "__Spotify": "spotify",
                    "__Warface": "warface",
                    "__Tag": "tag",
                    "__Categories": "categories",
                    "__Forums": "forums",
                    "__Pages": "pages",
                    "__Threads": "threads",
                    "__Posts": "posts",
                    "__Tags": "tags",
                    "__Users": "users",
                    "__Profile_posts": "profile_posts",
                    "__Conversations": "conversations",
                    "__Notifications": "notifications",
                    "__Search": "search",
                    "__Oauth": "oauth",
                    "__Posts_comments": "comments",
                    "__Contests": "contests",
                    "__Arbitrage": "arbitrage",
                    "__Money": "money",
                    "__Upgrade": "upgrade",
                    "__Avatar": "avatar",
                    "__Profile_posts_comments": "comments",
                    "__Conversations_messages": "messages",
                }.items():
                    if bad_path in path:
                        path = path.replace(bad_path, cute_path)
                for scope in scopes:
                    scope_parsed = scope.split("?")
                    joined = ", ".join(scope_parsed)
                    if len(scope_parsed) > 1:
                        if not any(_scope in self._scopes for _scope in scope_parsed):
                            logging.warning(msg=f'{Exceptions.MISSING_SCOPE.__name__}: One of [{joined}] scope is required to use "{path}" but not provided in your token.\nYou should recreate token with at least one of these scopes.', stack_info=False,)
                        elif scope_parsed[0] not in self._scopes:
                            logging.warning(msg=f'{Exceptions.MISSING_SCOPE.__name__}: "{joined}" scope is required to use "{path}" but not provided in your token.\nYou should recreate token with "{joined}" scope.', stack_info=False,)

        #  Огромная ебанина, которая находит код функции, парсит и выполняет его
        func_code = str(inspect.getsource(func)).replace(" -> httpx.Response", "")
        func_code = func_code.split("):\n", 1)[1]
        lines = func_code.split("\n")
        indent = lines[0].split('"""')[0]
        for line in lines.copy():
            if " def " in line:
                lines.remove(line)
        lines = [line.replace(indent, "", 1) for line in lines.copy()]
        return_code = "\n".join(lines).split('"""')[2].split("return ")[-1]
        func_code = "\n".join(lines).split('"""')[2].split("return ")[0]

        exec(func_code, globals(), loc)

        # Получаем переменные из локали, в которой отработал exec
        path = loc.get("path")
        params = loc.get("params", {})
        files = loc.get("files")  # Если вы файлы передаете в batch, то вы плохие люди
        dataJ = loc.get("dataJ", {})

        # Тримаем NONE
        params = _MainTweaks._TrimNONE(params)
        if type(dataJ) is dict:
            dataJ = _MainTweaks._TrimNONE(dataJ)

        if CREATE_JOB:  # Фикс для job'а
            path = self.base_url + path  # Полный путь кушает
            if dataJ:
                params.update(dataJ)  # Фикс для батча
            params["locale"] = self._locale  # Ставим локаль для ответа job'a

        # Ебанина для получения метода
        method = [eval(i.replace("method=", "").strip()) for i in return_code.split(",") if "method=" in i][0]
        return {"self": self, "path": path, "method": method, "params": params, "files": files, "dataJ": dataJ}

    def _AsyncExecutor(coroutine):
        #  Закостылил по гайду от величайшего
        #  https://www.youtube.com/watch?v=dQw4w9WgXcQ
        executor = ThreadPoolExecutor()
        result = executor.submit(asyncio.run, coroutine).result()
        return result

    class _Custom:
        def __init__(self):
            self.__params = {}
            self.__dataJ = {}
            self.__headers = {}

        @property
        def params(self):
            return self.__params

        @params.setter
        def params(self, value):
            if type(value) is dict:
                self.__params = value
            else:
                _WarningsLogger.warning(" Params must be dict")

        @property
        def json(self):
            return self.__dataJ

        @json.setter
        def json(self, value):
            if type(value) is str:
                self.__dataJ = json.loads(value)
            elif type(value) is dict:
                self.__dataJ = value
            else:
                _WarningsLogger.warning(" Json must be str or dict")

        @property
        def headers(self):
            return self.__headers

        @headers.setter
        def headers(self, value):
            if type(value) is dict:
                self.__headers = value
            else:
                _WarningsLogger.warning(" Headers must be dict")

        def reset(self):
            self.__params = {}
            self.__dataJ = {}
            self.__headers = {}


class DelaySync:
    def __init__(self, apis: list = []):
        self.apis = [i for i in apis]
        self.lock = Lock()
        self.shared_value = Value(ctypes.c_longdouble, 0)
        if apis:
            self.shared_value.value = max(
                [api_._auto_delay_time for api_ in apis] + [self.shared_value.value])
        for api in self.apis:
            api._add_delay_synchronizer(self)
            api._auto_delay_time = self.shared_value
            api._lock = self.lock

    def _synchronize(self, auto_delay_new):
        for api in self.apis:
            api._auto_delay_time.value = auto_delay_new

    def add(self, api):
        if type(api) is list:
            if api:
                self.shared_value.value = max(
                    [api_._auto_delay_time for api_ in api] + [self.shared_value.value])
            for api_ in api:
                self.apis.append(api_)
                api_._add_delay_synchronizer(self)
                api._auto_delay_time = self.shared_value
                api._lock = self.lock

        elif type(api) is dict:
            if api:
                self.shared_value.value = max(
                    [api_._auto_delay_time for api_ in api.values()] + [self.shared_value.value])
            for value in api.values():
                self.apis.append(value)
                value._add_delay_synchronizer(self)
                value._auto_delay_time = self.shared_value
                value._lock = self.lock
        else:
            self.shared_value.value = max(
                [self.shared_value.value, api._auto_delay_time])
            self.apis.append(api)
            api._add_delay_synchronizer(self)
            api._auto_delay_time = self.shared_value
            api._lock = self.lock

    def remove(self, api):
        if type(api) is list:
            for api_ in api:
                self.apis.remove(api_)
                api_._remove_delay_synchronizer(self)
        elif type(api) is dict:
            for value in api.values():
                self.apis.remove(value)
                value._remove_delay_synchronizer(self)
        else:
            self.apis.remove(api)
            api._remove_delay_synchronizer()

    def clear(self):
        apis_td = []
        for api in self.apis:
            apis_td.append(api)
        for api in apis_td:
            self.remove(api)

    def __del__(self,):  # Удаляем линки на синхронайзеры из объектов Forum/Market при удалении
        self.clear()


class Debug:
    def __init__(self):
        self._status = False
        self.DebugLogger = _DebugLogger
        self.DebugLogger.setLevel(level=logging.DEBUG)

        __DebugHandler = logging.FileHandler(filename=f"{os.path.basename(sys.argv[0])} # LOLZTEAM {time.strftime('%Y.%m.%d - %H-%M-%S')}.log", mode="w", encoding="UTF-8")
        formatter = logging.Formatter("%(asctime)s | %(message)s")
        __DebugHandler.setFormatter(formatter)
        self.DebugHandler = __DebugHandler

    @property
    def status(self):
        return self._status

    def enable(self):
        if not self._status:
            self._status = True
            self.DebugLogger.addHandler(self.DebugHandler)
            _DebugLogger.debug(f"Debug started | LOLZTEAM {version('LOLZTEAM')}")
        else:
            _WarningsLogger.warning(" Debug logger already enabled")

    def disable(self):
        if self._status:
            self._status = False
            self.DebugLogger.removeHandler(self.DebugHandler)
            _DebugLogger.debug(f"Debug stopped | LOLZTEAM {version('LOLZTEAM')}")
        else:
            _WarningsLogger.warning(" Debug logger already disabled")


def CreateJob(func, job_name, **cur_kwargs) -> dict:
    CREATE_JOB = True
    code_data = _MainTweaks._AsyncExecutor(_MainTweaks._ExecCode(func=func, loc=locals(), cur_kwargs=cur_kwargs))
    job = {"id": str(job_name), "uri": code_data["path"], "method": code_data["method"], "params": code_data["params"]}
    return job


async def SendAsAsync(func, **cur_kwargs) -> httpx.Response:
    """
    Send async request

    :param func: Target function
    :param kwargs: Target function parameters

    :return: httpx Response object
    """
    SEND_AS_ASYNC = True
    code_data = await _MainTweaks._ExecCode(func=func, loc=locals(), cur_kwargs=cur_kwargs)
    from .API import _send_async_request
    return await _send_async_request(self=code_data["self"], method=code_data["method"], path=code_data["path"], params=code_data["params"], dataJ=code_data["dataJ"], files=code_data["files"])



from importlib.metadata import version
import httpx
import os

current_version = version("LOLZTEAM")
response = httpx.get("https://pypi.org/pypi/LOLZTEAM/json")
newest_version = response.json()["info"]["version"]
if current_version != newest_version:
    venv_folder = None
    for root, dirs, files in os.walk(os.curdir):
        if all(folder in dirs for folder in ["Lib", "Scripts", "Include"]):
            if "pyvenv.cfg" in files:
                venv_folder = root
                print(f"Found venv - {venv_folder}")
                break
    print(f"Updating LOLZTEAM from {current_version} to {newest_version}")
    import subprocess
    import sys
    from importlib import reload
    if venv_folder:
        subprocess.call(
            f"{venv_folder}\\Scripts\\pip install LOLZTEAM --upgrade",
            shell=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.STDOUT,
        )
    else:
        subprocess.call(
            "pip install LOLZTEAM --upgrade",
            shell=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.STDOUT,
        )
        reload(sys.modules["LOLZTEAM"])
        print(f"LOLZTEAM updated to {newest_version}")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements = ["pydantic", "httpx", "httpx[http2]", "httpx[socks]"]

setuptools.setup(
    name="LOLZTEAM",
    version="1.4.7",
    author="AS7RID",
    author_email="as7ridwork@gmail.com",
    description="A library that contains all the methods of the Lolzteam API (Market/Forum/Antipublic)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AS7RIDENIED/LOLZTEAM",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    classifiers=["Programming Language :: Python :: 3.11"],
    python_requires='>=3.9'
)



class AS7RID_FUCK_UP(Exception):
    pass


class INVALID_PROXY_TYPE(Exception):
    pass


class USERID_REQUIRED_FOR_BATCH(Exception):
    pass


class INVALID_ARG(Exception):
    pass


class UNEXPECTED_ARG(Exception):
    pass


class URL_IS_DIFFERENT_FROM_BASE_MARKET(Exception):
    pass


class MISSING_SCOPE(Exception):
    pass


class BAD_TOKEN(Exception):
    pass

class PATH_NOT_DEFINED(Exception):
    pass